@@ -2,17 +2,23 @@
 #this repository contains the full copyright notices and license terms.
 {
     'name': 'Account Statement',
+    'name_bg_BG': 'Отчет на сметка',
     'name_de_DE': 'Buchhaltung Bankauszüge',
     'name_es_CO': 'Estado de Cuentas',
     'name_es_ES': 'Estado de cuentas',
     'name_fr_FR': 'Relevé comptable',
-    'version': '1.8.2',
+    'name_nl_NL': 'Bankafschriften',
+    'version': '2.0.1',
     'author': 'B2CK',
     'email': 'info@b2ck.com',
     'website': 'http://www.tryton.org/',
     'description': '''Financial and Accounting Module with:
     - Statement
     - Statement journal
+''',
+    'description_bg_BG': '''Финансов и счетоводен модул с:
+    - Отчети
+    - Дневник на отчети
 ''',
     'description_de_DE': '''Modul für Buchhaltung und Bankauszüge mit
     - Abstimmung von Bankauszügen und Rechnungen
@@ -29,6 +35,10 @@
     - Relevé
     - Journal de relevés
     ''',
+    'description_nl_NL': '''Module voor het verwerken van bankafschriften met:
+    - bankboeken
+    - afletteren van rekeningen
+''',
     'depends': [
         'account',
         'company',
@@ -42,9 +52,11 @@
         'invoice.xml',
         ],
     'translation': [
+        'bg_BG.csv',
         'de_DE.csv',
         'es_CO.csv',
         'es_ES.csv',
         'fr_FR.csv',
+        'nl_NL.csv',
     ],
 }
